import click


class MultipleChoice(click.Choice):
    def convert(self, value, param, ctx):
        values = [v.strip() for v in value.split(",")]
        normed_values = list()
        for v in values:
            normed_values.append(super().convert(v, param, ctx))
        return normed_values

    def get_metavar(self, param: click.ParamType) -> str:
        choices_str = ",".join(self.choices)
        return f"[{choices_str}]"



from pathlib import Path

from pagesaver.lib.notion import block, property
from pagesaver.lib.notion.client import NotionClient, NotionUnofficialClient
from pagesaver.settings import pagesaver_settings

TITLE_KEY = pagesaver_settings.TITLE_PROPERTY
LINK_KEY = pagesaver_settings.LINK_PROPERTY
MHTML_KEY = pagesaver_settings.MHTML_PROPERTY


def create_page(database_id: str, token: str, title: str, link: str) -> str:
    client = NotionClient(token=token)
    properties = {
        TITLE_KEY: property.Title(children=[property.Text(content=title)]).to_dict(),
        LINK_KEY: property.Url(url=link).to_dict(),
    }

    return client.create_page(database_id, properties)["id"]


def upload_file(token_v2: str, block_id: str, file_path: Path):
    unofficial_client = NotionUnofficialClient(token_v2=token_v2)
    return unofficial_client.upload_file(block_id, file_path)


def add_mhtml_property(page_id: str, token: str, token_v2: str, file_path: Path):
    url = upload_file(token_v2, page_id, file_path)
    client = NotionClient(token=token)
    client.update_page_properties(
        page_id,
        {
            MHTML_KEY: property.Files(
                children=[property.NotionHostedFile(name=file_path.name, url=url)]
            ).to_dict(),
        },
    )


def add_pdf_block(page_id: str, token: str, token_v2: str, file_path: Path):
    client = NotionClient(token=token)
    block_id = client.add_block(
        page_id, [block.PDFBlock(url="https://www.notion.so/placeholder.pdf").to_dict()]
    )["results"][0]["id"]

    url = upload_file(token_v2, block_id, file_path)

    payload = block.PDFBlock(url=url).to_dict()
    del payload["pdf"]["type"]
    client.update_block(block_id, payload)
    return block_id


if __name__ == "__main__":
    import os

    database_id = os.getenv("NOTION_DATABASE_ID")
    token = os.getenv("NOTION_API_TOKEN")
    token_v2 = os.getenv("NOTION_TOKEN_V2")
    page_id = create_page(
        database_id, title="test", token=token, link="https://www.notion.so"
    )

    add_mhtml_property(page_id, token, token_v2, Path("exported.mhtml"))
    add_pdf_block(page_id, token, token_v2, Path("exported.pdf"))



# https://developers.notion.com/reference/page-property-values#type-objects

from pydantic import BaseModel


class Property(BaseModel):
    id: str = ""
    type: str

    def get_objects(self):
        return self.__dict__.get(self.type, None)

    def to_dict(self, include_meta=False, include_type=True):
        data = {str(self.type): self.get_objects()}
        if include_meta:
            data.update({"id": self.id, "type": self.type})
        if include_type:
            data.update({"type": self.type})
        return data


class ChildrenMixin:
    children: list

    def get_objects(self):
        return [child.to_dict(include_type=True) for child in self.children]


class Text(Property):
    type: str = "text"
    content: str

    def get_objects(self):
        return {"content": self.content}


class Title(ChildrenMixin, Property):
    type: str = "title"
    children: list[Text]


class Url(Property):
    type: str = "url"
    url: str


class File(Property):
    type: str = "external"
    name: str
    url: str

    def get_objects(self):
        return {
            "name": self.name,
            "type": self.type,
            self.type: {"url": self.url},
        }

    def to_dict(self, include_meta=False, include_type=False):
        return super().to_dict(include_meta, include_type).get(self.type)


class NotionHostedFile(File):
    type: str = "file"


class Files(ChildrenMixin, Property):
    type: str = "files"
    children: list[File]


if __name__ == "__main__":
    print(Url(url="https://www.notion.so").to_dict())
    print(Text(content="hello").to_dict())
    print(Title(children=[Text(content="hello")]).to_dict())
    print(Files(children=[File(name="hello", url="https://www.notion.so")]).to_dict())
    print(
        Files(
            children=[NotionHostedFile(name="hello", url="https://www.notion.so")]
        ).to_dict()
    )



from functools import cached_property

from django.db import models

from pagesaver.enums import ExportFormat
from pagesaver.lib.base.models import BaseModelWithUUID


class Snapshot(BaseModelWithUUID):
    url = models.URLField(db_index=True)
    title = models.CharField(max_length=512, default="", blank=True, db_index=True)

    @cached_property
    def size(self):
        pass

    class Meta:
        verbose_name = "快照"
        verbose_name_plural = verbose_name
        db_table = "snapshot"

    def __str__(self):
        return self.title

    def __repr___(self):
        return "<Snapshot: {}|{}>".format(self.title, self.uuid)


class SnapshotResult(BaseModelWithUUID):
    FORMAT_CHOICES = (
        (ExportFormat.PDF.value, "PDF"),
        (ExportFormat.MHTML.value, "MHTML文件"),
    )
    snapshot = models.ForeignKey(
        Snapshot, on_delete=models.CASCADE, related_name="results"
    )
    format = models.CharField(max_length=10, choices=FORMAT_CHOICES)
    path = models.FileField()

    class Meta:
        unique_together = ("snapshot", "format")
        verbose_name = "快照结果"
        verbose_name_plural = verbose_name
        db_table = "snapshot_result"

    def __str__(self):
        return str(self.path)

    def __repr___(self):
        return "<SnapshotResult: {} |{}>".format(self.snapshot.title, self.uuid)



# Generated by Django 5.0.7 on 2024-08-02 07:19

import uuid

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Snapshot',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('updated', models.DateTimeField(auto_now=True, db_index=True)),
                ('uuid', models.UUIDField(db_index=True, default=uuid.uuid4, unique=True)),
                ('url', models.URLField(db_index=True)),
                ('title', models.CharField(blank=True, db_index=True, default='', max_length=512)),
            ],
            options={
                'verbose_name': '快照',
                'verbose_name_plural': '快照',
                'db_table': 'snapshot',
            },
        ),
        migrations.CreateModel(
            name='SnapshotResult',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('updated', models.DateTimeField(auto_now=True, db_index=True)),
                ('uuid', models.UUIDField(db_index=True, default=uuid.uuid4, unique=True)),
                ('format', models.CharField(choices=[('PDF', 'PDF'), ('MHTML', 'MHTML文件')], max_length=10)),
                ('path', models.FileField(upload_to='')),
                ('snapshot', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='record.snapshot',related_name='results')),
            ],
            options={
                'verbose_name': '快照结果',
                'verbose_name_plural': '快照结果',
                'db_table': 'snapshot_result',
                'unique_together': {('snapshot', 'format')},
            },
        ),
    ]



import importlib.util
from pathlib import Path

DEFAULTS = {
    "STORAGE": {
        "type": "local",
        "path": "./var",
    },
    "TITLE_PROPERTY": "title",
    "LINK_PROPERTY": "link",
    "MHTML_PROPERTY": "mhtml",
    "SERVER_BIND": "0.0.0.0:8001",
}


class Setting:
    def __init__(self, user_settings=None, defaults=None) -> None:
        self.user_settings = user_settings or {}
        self.defaults = defaults or {}

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError("Invalid pagesaver setting: '%s'" % attr)
        try:
            val = self.user_settings[attr]
        except KeyError:
            val = self.defaults[attr]
        setattr(self, attr, val)
        return val

    @classmethod
    def from_object(cls, instance, defaults=None):
        user_settings = {
            key: getattr(instance, key)
            for key in dir(instance)
            if not key.startswith("_")
        }
        return cls(user_settings=user_settings, defaults=defaults)

    @classmethod
    def from_pyfile(cls, filename, defaults=None):
        file_path = Path(filename)
        spec = importlib.util.spec_from_file_location("module.name", file_path)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
        return cls.from_object(module, defaults)


class LazySetting:
    def __init__(self) -> None:
        self._wrapped = None

    def _setup(self):
        config_path = Path("config.py").absolute()  # Todo
        if config_path.exists():
            self._wrapped = Setting.from_pyfile(config_path, defaults=DEFAULTS)
        else:
            self._wrapped = Setting(user_settings={}, defaults=DEFAULTS)

    def __getattr__(self, attr):
        if self._wrapped is None:
            self._setup()
        return getattr(self._wrapped, attr)


pagesaver_settings = LazySetting()



from pydantic import BaseModel


class Block(BaseModel):
    id: str = ""
    type: str

    def get_objects(self):
        return self.__dict__.get(self.type, None)

    def to_dict(self, include_meta=False, include_type=True):
        data = {str(self.type): self.get_objects()}
        if include_meta:
            data.update({"id": self.id, "type": self.type})
        if include_type:
            data.update({"type": self.type})
        return data


class PDFBlock(Block):
    type: str = "pdf"
    url: str
    external: bool = True

    def get_objects(self):
        pdf_type = "external" if self.external else "file"
        return {
            "type": pdf_type,
            pdf_type: {"url": self.url},
        }


if __name__ == "__main__":
    print(PDFBlock(url="https://www.notion.so/placeholder.pdf").to_dict())



from pathlib import Path
from typing import TypeVar

from playwright.sync_api import Page, sync_playwright

from pagesaver.enums import ExportFormat


def _export_pdf(page: Page, path: Path) -> str:
    pdf_path = path.with_suffix(".pdf").as_posix()
    page.emulate_media(media="screen")
    page.pdf(path=pdf_path)
    return pdf_path


def _export_mhtml(page: Page, path: Path) -> str:
    mhtml_path = path.with_suffix(".mhtml")
    client = page.context.new_cdp_session(page)
    data = client.send("Page.captureSnapshot", {"format": "mhtml"})
    mhtml_path.write_bytes(data["data"].encode("utf-8"))
    return mhtml_path.as_posix()


T = TypeVar("T", bound=ExportFormat)


def export(
    url: str,
    path: Path,
    formats: list[T],
    headless: bool = True,
    proxy: str | None = None,
    **browser_decls,
):
    """
    Args:
        url (str): The url you want to export
        path (Path): The directory of the exported file
        formats (List[T]): The format you want to export
        headless (bool, optional): Whether to use headless mode. Defaults to True.
        browser_decls (Dict, optional): The browser declaration. Defaults to {}.
    Return:
        (list[dict[str]], dict[str]): The path list and the page information
    """
    if proxy:
        browser_decls["proxy"] = {"server": proxy}
    with sync_playwright() as playwright:
        browser = playwright.chromium.launch(headless=headless)

        context = browser.new_context(**browser_decls)
        page = context.new_page()
        page.goto(url)

        page.wait_for_load_state("networkidle")
        info = {
            "title": page.title(),
        }
        # Wait img
        locators = page.locator("img").all()
        for locator in locators:
            locator.evaluate("node => node.scrollIntoView()")
        page.evaluate("() => window.scrollTo(0, document.body.scrollHeight)")
        page.wait_for_load_state("networkidle")

        # Back to top
        page.evaluate("() => window.scrollTo(0, 0)")

        path_lst = list()
        # pdf
        if ExportFormat.PDF in formats:
            path_lst.append(
                {
                    "format": ExportFormat.PDF.value,
                    "path": _export_pdf(page, path),
                }
            )

        # mhtml
        if ExportFormat.MHTML in formats:
            path_lst.append(
                {
                    "format": ExportFormat.MHTML.value,
                    "path": _export_mhtml(page, path),
                }
            )

        browser.close()
        return (path_lst, info)


if __name__ == "__main__":
    export(
        "https://www.baidu.com",
        Path("").absolute() / "test",
        [ExportFormat.PDF],
    )



from ninja import NinjaAPI, errors
from ninja.security import APIKeyHeader

from pagesaver.authorization.models import APIToken
from pagesaver.record.api import router as record_router


class APIKey(APIKeyHeader):
    param_name = "Authorization"

    def authenticate(self, request, key):
        if token := APIToken.objects.filter(token=key).first():
            if token.verify():
                return token
        raise errors.AuthenticationError


api = NinjaAPI(auth=APIKey())

api.add_router("/record/", record_router)



# Generated by Django 5.0.7 on 2024-08-12 05:28

from django.db import migrations, models

import pagesaver.authorization.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='APIToken',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('updated', models.DateTimeField(auto_now=True, db_index=True)),
                ('expired', models.DateTimeField(default=pagesaver.authorization.models._default_expire_time, verbose_name='token expire time')),
                ('token', models.TextField(blank=True, verbose_name='token')),
            ],
            options={
                'verbose_name': 'api token',
                'verbose_name_plural': 'api token',
                'db_table': 'api_token',
            },
        ),
    ]



class BaseRunner:

    def __init__(self) -> None:
        self.tasks = list()

    def add_task(self, func, *args, **kwargs):
        self.tasks.append((func, args, kwargs))

    def start(self):
        raise NotImplementedError

    def stop(self):
        raise NotImplementedError



import uuid

from django.db import models


class BaseModel(models.Model):
    created = models.DateTimeField(auto_now_add=True, db_index=True)
    updated = models.DateTimeField(auto_now=True, db_index=True)

    class Meta:
        abstract = True


class BaseModelWithUUID(BaseModel):
    uuid = models.UUIDField(unique=True, db_index=True, default=uuid.uuid4)

    class Meta:
        abstract = True



from datetime import timedelta

import click
from django.core.management import call_command
from django.utils import timezone

from pagesaver import __version__


@click.command("init")
@click.help_option("-h", "--help")
def init_command(*args, **kwargs):
    """Initialize PageSaver"""
    click.secho(f"[+] Initializing PageSaver v{__version__}...", fg="green")
    click.secho("{}\n".format("-" * 30), fg="green")

    click.secho("[+] Creating database and running initial migrations...", fg="green")
    call_command("migrate")

    click.secho("\n{}".format("-" * 30), fg="green")
    click.secho("[√] Init successfully.", fg="green")

    from pagesaver.authorization.models import APIToken

    expired = timezone.now().astimezone() + timedelta(days=365 * 100)
    token = APIToken.objects.create(expired=expired)
    click.echo(
        "Your API token: {token} (expired: {expired})".format(
            token=click.style(token.token, fg="blue"),
            expired=click.style(expired.strftime("%Y-%m-%d %H:%M:%S"), fg="red"),
        )
    )

    click.echo(
        """
{hint} To using PageSaver, Please install dependencies with the following command: 
    playwright install # browsers 
    playwright install-deps # dependencies to run browsers""".format(
            hint=click.style("Hint:", fg="magenta")
        )
    )

    click.echo(
        """
{hint} To start PageSaver, run: 
    pagesaver server # then visit http://127.0.0.1:8000
        """.format(
            hint=click.style("Hint:", fg="magenta")
        )
    )



import click

from pagesaver import __version__

VERSION = rf"""
        ____                  _       _   _       _
        |  _ \ __ _  __ _  ___| |__   | | | |_   _| |__
        | |_) / _` |/ _` |/ _ \ '_ \  | |_| | | | | '_ \
        |  __/ (_| | (_| |  __/ | | | |  _  | |_| | |_) |
        |_|   \__,_|\__, |\___|_| |_| |_| |_|\__,_|_.__/
                    |___/

                    VERSION {__version__}
"""


@click.group()
@click.version_option(__version__, "-V", "--version", message=VERSION)
@click.help_option("-h", "--help")
def main():
    pass


from .export import export_command
from .init import init_command
from .server import server_command

main.add_command(export_command)
main.add_command(server_command)
main.add_command(init_command)


from pathlib import Path
from typing import Optional

from pagesaver.constants import SUPPORT_FORMATS
from pagesaver.enums import ExportFormat
from pagesaver.record.models import Snapshot, SnapshotResult
from pagesaver.settings import pagesaver_settings
from pagesaver.utils import export_utils, notion_utils
from pagesaver.utils.datetime_utils import get_now_str

STORAGE = pagesaver_settings.STORAGE


def export_task(snapshot_id: int, url: str, format: list[str]):
    snapshot = Snapshot.objects.get(pk=snapshot_id)

    format = None or SUPPORT_FORMATS
    base_directory = Path(STORAGE["path"]).absolute()
    directory = base_directory / str(snapshot.uuid) / get_now_str()

    path_lst, info = export_utils.export(
        url, directory, [ExportFormat(i) for i in format]
    )
    for item in path_lst:
        path = str(Path(item["path"]).relative_to(base_directory))
        SnapshotResult.objects.create(
            snapshot=snapshot, format=item["format"], path=path
        )
    snapshot.title = info.get("title", "")
    snapshot.save(update_fields=["title"])

    return path_lst, info


def notion_push_task(
    snapshot_id: int,
    url: str,
    *,
    api_token: str,
    database_id: str,
    token_v2: Optional[str] = None,
    title: str = "",
    **kwargs,
):
    snapshot = Snapshot.objects.get(pk=snapshot_id)
    path_lst = list(snapshot.results.all().values("format", "path"))

    title = title or snapshot.title

    page_id = notion_utils.create_page(
        database_id, title=title, token=api_token, link=url
    )
    if token_v2:
        base_directory = Path(STORAGE["path"]).absolute()
        path_2_dict = {p["format"]: (base_directory / p["path"]) for p in path_lst}
        if path := path_2_dict.get(ExportFormat.MHTML.value):
            notion_utils.add_mhtml_property(page_id, api_token, token_v2, path)
        if path := path_2_dict.get(ExportFormat.PDF.value):
            notion_utils.add_pdf_block(page_id, api_token, token_v2, path)

    return page_id


if __name__ == "__main__":
    import os

    from pagesaver.record.models import Snapshot

    url = "https://www.baidu.com"
    snapshot = Snapshot.objects.create(url=url)
    export_task(snapshot.id, url, ["PDF"])
    notion_push_task(
        snapshot.id,
        api_token=os.environ["NOTION_API_TOKEN"],
        database_id=os.environ["NOTION_DATABASE_ID"],
        token_v2=os.environ["NOTION_TOKEN_V2"],
        title="test",
    )



import threading

from pagesaver.record.runner.base import BaseRunner


class BackgroundThreadRunner(BaseRunner):
    def __init__(self) -> None:
        super().__init__()
        self.thread = threading.Thread(target=self._run)
        self.thread.daemon = True

    def _run(self):
        while len(self.tasks) > 0:
            func, args, kwargs = self.tasks.pop(0)
            func(*args, **kwargs)

    def start(self):
        self.thread.start()

    def stop(self):
        pass  # SIGINT


if __name__ == "__main__":
    runner = BackgroundThreadRunner()
    runner.add_task(lambda: print("hello"))
    runner.start()



import asyncio

import click
from hypercorn.asyncio import serve
from hypercorn.config import Config

from pagesaver.core.asgi import application
from pagesaver.settings import pagesaver_settings


@click.command("server")
@click.help_option("-h", "--help")
@click.option(
    "-b",
    "--bind",
    "bind",
    show_default=True,
    default=pagesaver_settings.SERVER_BIND,
    help="The TCP host/address to bind to.",
)
def server_command(*args, **kwargs):
    """Run PageSaver HTTP server"""
    config = Config.from_mapping({"bind": pagesaver_settings.SERVER_BIND}, **kwargs)
    asyncio.run(serve(application, config))



from urllib.parse import urljoin

import requests


class BaseClient:
    def __init__(self, base_url) -> None:
        self.base_url = base_url
        self._client = None

    @property
    def client(self):
        return self._client

    def build_url(self, path, *args, **kwargs):
        raise NotImplementedError

    def build_headers(self, *args, **kwargs):
        raise NotImplementedError

    def do_request(self, method, url, headers=None, query=None, body=None, **kwargs):
        raise NotImplementedError

    def parse_response(self, response):
        raise NotImplementedError

    def make_request(self, method, path, query=None, body=None, **kwargs):
        url = self.build_url(path, query=query, body=body, **kwargs)
        headers = self.build_headers()
        return self.do_request(
            method, url, headers=headers, query=query, body=body, **kwargs
        )

    def request(self, method, path, query=None, body=None, **kwargs):
        response = self.make_request(method, path, query=query, body=body, **kwargs)
        return self.parse_response(response)

    def get(self, path, query=None, **kwargs):
        return self.request("GET", path, query=query, **kwargs)

    def post(self, path, body, **kwargs):
        return self.request("POST", path, body=body, **kwargs)

    def patch(self, path, body, **kwargs):
        return self.request("PATCH", path, body=body, **kwargs)

    def put(self, path, body, **kwargs):
        return self.request("PUT", path, body=body, **kwargs)

    def delete(self, path, **kwargs):
        return self.request("DELETE", path, **kwargs)


class RequestsBaseClient(BaseClient):
    DEFAULT_TIMEOUT = 5

    def __init__(self, base_url, timeout=DEFAULT_TIMEOUT) -> None:
        super().__init__(base_url=base_url)
        self._client = requests.Session()
        self.timeout = timeout

    def build_url(self, path, **kwargs):
        return urljoin(self.base_url, path)

    def do_request(self, method, url, headers=None, query=None, body=None, **kwargs):
        return requests.request(
            method,
            url,
            timeout=kwargs.pop("timeout", self.timeout),
            headers=headers,
            params=query,
            json=body,
            **kwargs,
        )

    def parse_response(self, response: requests.Response):
        response.raise_for_status
        if response.status_code != requests.codes.ok:
            raise requests.HTTPError(
                f"{response.status_code} {response.reason} for {response.url} {response.text}"
            )
        return response.json()



from django.contrib import admin

from .models import Snapshot, SnapshotResult


class SnapshotAdmin(admin.ModelAdmin):
    list_display = ("uuid", "title", "url", "size", "created")
    fields = ("uuid", "title", "url")
    readonly_fields = ("uuid",)
    list_filter = ("created",)


class SnapshotResultAdmin(admin.ModelAdmin):
    list_display = ("uuid", "snapshot", "format", "path", "created")
    readonly_fields = ("uuid", "snapshot", "format", "created")
    fields = ("uuid", "created", "snapshot", "format", "path")
    list_filter = ("format", "snapshot", "created")


admin.site.register(Snapshot, SnapshotAdmin)
admin.site.register(SnapshotResult, SnapshotResultAdmin)



from pathlib import Path

import click

from pagesaver.cli.types import MultipleChoice
from pagesaver.constants import SUPPORT_FORMATS
from pagesaver.enums import ExportFormat
from pagesaver.utils import export_utils

SUCCESS_ANSI = click.style("successfully", fg="green")


@click.command("export")
@click.argument("url", type=click.STRING)
@click.option(
    "-f",
    "--format",
    "format",
    required=True,
    type=MultipleChoice(SUPPORT_FORMATS, case_sensitive=False),
    help="Format which you want to export",
)
@click.option(
    "-o",
    "--output",
    "output",
    required=True,
    type=click.Path(writable=True, dir_okay=True, file_okay=False),
    help="Output directory of the file",
)
@click.option(
    "-n",
    "--name",
    "name",
    required=False,
    default="exported",
    show_default=True,
    type=click.STRING,
    help="Name of the exported file",
)
@click.help_option("-h", "--help")
@click.pass_context
def export_command(ctx, url, format, output, name):
    """Export page to the output file"""
    path_lst, _ = export_utils.export(
        url, Path(output).absolute() / name, [ExportFormat(i) for i in format]
    )
    click.echo(f"Exported {SUCCESS_ANSI}:")
    for path in path_lst:
        click.echo(f"  - {path['format']}: {path['path']}")



from typing import Optional

from ninja import Field, Query, Router, Schema

from pagesaver.enums import ExportFormat
from pagesaver.record.models import Snapshot
from pagesaver.record.runner import BackgroundThreadRunner
from pagesaver.record.tasks import export_task, notion_push_task

router = Router()


class RecordFilters(Schema):
    format: list[ExportFormat] = Field(
        default=list(ExportFormat.__members__.values()),
    )


class NotionFilters(RecordFilters):
    api_token: str
    database_id: str
    token_v2: Optional[str] = None
    title: Optional[str] = ""


@router.get("/notion/{path:url}")
def notion_record(request, url: str, filters: Query[NotionFilters]):
    snapshot = Snapshot.objects.create(url=url)

    runner = BackgroundThreadRunner()
    runner.add_task(export_task, snapshot.pk, url, format=filters.format)
    runner.add_task(
        notion_push_task, snapshot.pk, url, **filters.model_dump(mode="json")
    )
    runner.start()
    return {
        "url": url,
        "uuid": snapshot.uuid,
        "params": filters.model_dump(mode="json"),
    }


@router.get("/{path:url}")
def record(request, url: str, filters: Query[RecordFilters]):
    snapshot = Snapshot.objects.create(url=url)
    runner = BackgroundThreadRunner()
    runner.add_task(export_task, snapshot.pk, url, **filters.model_dump(mode="json"))
    runner.start()
    return {
        "url": url,
        "uuid": snapshot.uuid,
        "params": filters.model_dump(mode="json"),
    }



import binascii
import os
from datetime import timedelta

from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from pagesaver.lib.base.models import BaseModel


def _default_expire_time():
    return timezone.now() + APIToken.TOKEN_EXPIRE


class APIToken(BaseModel):
    TOKEN_LENGTH = 32
    TOKEN_EXPIRE = timedelta(days=30)

    expired = models.DateTimeField(_("token expire time"), default=_default_expire_time)
    token = models.TextField(_("token"), blank=True)

    def generate_key(self):
        return binascii.hexlify(os.urandom(int(self.TOKEN_LENGTH / 2))).decode()

    def verify(self):
        return self.check_exp() and len(self.token) == self.TOKEN_LENGTH

    def check_exp(self):
        return timezone.now() < self.expired

    def refresf_exp(self):
        self.expired = timezone.now() + self.TOKEN_EXPIRE
        self.save(update_fields=["expired"])

    def save(self, *args, **kwargs):
        if not self.token:
            self.token = self.generate_key()
        super().save(*args, **kwargs)
    class Meta:
        verbose_name = _("api token")
        verbose_name_plural = verbose_name
        db_table = "api_token"



import mimetypes
import os
from pathlib import Path

from pagesaver.lib.base.client import RequestsBaseClient

API_BASE_URL = "https://api.notion.com/v1/"
UNOFFICIAL_BASE_URL = "https://www.notion.so/api/v3/"


class NotionClient(RequestsBaseClient):
    def __init__(self, token, base_url=None, **kwargs) -> None:
        super().__init__(base_url=base_url or API_BASE_URL, **kwargs)
        self.token = token or os.getenv("NOTION_API_TOKEN")

    def build_headers(self):
        return {
            "Authorization": f"Bearer {self.token}",
            "Content-Type": "application/json",
            "Notion-Version": "2022-06-28",
        }

    def create_page(self, database_id, properties):
        payload = {
            "parent": {
                "database_id": database_id,
                "type": "database_id",
            },
            "properties": properties,
            "children": [],
        }
        return self.post("pages", body=payload)

    def add_block(self, page_id, children):
        payload = {
            "children": children,
        }
        return self.patch(f"blocks/{page_id}/children", body=payload)

    def update_block(self, block_id, payload):
        return self.patch(f"blocks/{block_id}", body=payload)

    def update_page_properties(self, page_id, properties):
        payload = {
            "properties": properties,
        }
        return self.patch(f"pages/{page_id}", body=payload)


class NotionUnofficialClient(RequestsBaseClient):

    def __init__(self, token_v2, space_id=None, base_url=None, **kwargs) -> None:
        super().__init__(base_url=base_url or UNOFFICIAL_BASE_URL, **kwargs)
        self.token_v2 = token_v2 or os.getenv("NOTION_TOKEN_V2")
        self.space_id = space_id or self._get_space_info().get("spaceId")

    def _get_space_info(self):
        records = self.post("loadUserContent", {})["recordMap"]
        return list(records["space_view"].values())[0]

    def build_headers(self):
        return {
            "cookie": f"token_v2={self.token_v2}",
            "Notion-Client-Version": "23.13.0.316",
            "content-type": "application/json",
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
        }

    def _get_upload_url(self, block_id, path: Path):
        payload = {
            "bucket": "secure",
            "name": path.name,
            "contentType": mimetypes.guess_type(path)[0] or "text/plain; charset=utf-8",
            "record": {
                "table": "block",
                "id": block_id,
                "spaceId": self.space_id,
            },
            "supportExtraHeaders": True,
            "contentLength": path.stat().st_size,
        }

        return self.post("getUploadFileUrl", body=payload)

    def upload_file(self, block_id, file_path: Path):
        data = self._get_upload_url(block_id, file_path)
        self.do_request(
            "POST",
            data["signedUploadPostUrl"],
            data=data["fields"],
            headers={header["name"]: header["value"] for header in data["postHeaders"]},
            files={"file": file_path.open("rb")},
            timeout=10,
        )
        return data["url"]


if __name__ == "__main__":
    import os
    client = NotionUnofficialClient(token_v2=os.getenv("NOTION_TOKEN_V2"))
    print(client.space_id)
